"""
Tools for working with restful apis
"""
__author__ = 'Alistair Broomhead'
from slacken.dicttypes import AttrDict
from slacken.rest_access import RESTaccess
from slacken.xml_accessor import XMLAccessor
